" Vim Keymap file for hebrew
" Maintainer  : Ron Aaron <ron@ronware.org>
" Last Updated: Wed 28 Feb 2001 21:28:51
" This is the standard Israeli keyboard layout

" Use this short name in the status line.
let b:keymap_name = "heb"

loadkeymap
a	<char-249>	" ù - shin
b	<char-240>	" ð - nun
c	<char-225>	" á - bet
d	<char-226>	" â - gimel
e	<char-247>	" ÷ - qof
f	<char-235>	" ë - kaf
g	<char-242>	" ò - ayin
h	<char-233>	" é - yod
i	<char-239>	" ï - final nun
j	<char-231>	" ç - het
k	<char-236>	" ì - lamed
l	<char-234>	" ê - final kaf
m	<char-246>	" ö - tsadi
n	<char-238>	" î - mem
o	<char-237>	" í - final mem
p	<char-244>	" ô - pe
q	/		" / - slash
r	<char-248>	" ø - resh
s	<char-227>	" ã - dalet
t	<char-224>	" à - alef
u	<char-229>	" å - vav
v	<char-228>	" ä - he
w	'		" ' - single-quote
x	<char-241>	" ñ - samekh
y	<char-232>	" è - tet
z	<char-230>	" æ - zayin
,	<char-250>	" ú - tav
.	<char-245>	" õ - final tsadi
;	<char-243>	" ó - final pe
'	,		" , - comma
/	.		" . - period
`	;		" ; - semicolon
